function [X, W] = GaussQuadrature( g, dim)
    % The function returns the Gauss points x and weights w. 
    % The input q is the order of the quadrature and dim is the number of
    % spatial dimensions. 

    GaussPoint  = zeros(g^dim, dim) ;
    GaussWeight = zeros(g^dim, 1) ; 

    % Compute Gauss points and weights: 
    m           = g - 1;
    m1          = m + 1;
    m2          = m + 2;
    
    l           = linspace(-1, 1, m1)';
    y           = cos((2*(0:m)'+1)*pi/(2*m+2)) + (0.27/m1)*sin(pi*l*m/m2);
    y0          = 2;
    G           = zeros(m1, m2);
    dG          = zeros(m1, m2);
    
    while max(abs(y-y0)) > eps
        G(:,1)  = 1;
        dG(:,1) = 0;
        G(:,2)  = y;
        dG(:,2) = 1;
        
        for k = 2:m1
            G(:,k+1) = ( (2*k-1) * y .*G(:,k) - (k-1)*G(:,k-1) ) /k;
        end %k
        
        dG  = m2 * ( G(:,m1) - y .* G(:,m2) ) ./ (1 - y.^2);
        y0  = y;
        y   = y0 - G(:,m2)./dG;
    end %while
                       
    x = - y;
    w = 2./ ( (1-x.^2).*dG.^2 ) * (m2 / m1)^2;
    
    % Setting up points in several dimensions:
    n = 1;
    if ( dim == 1 )
        for i = 1:g
            GaussPoint(n,:) = [ x(i) ];
            GaussWeight(n)  = w(i);
            n = n+1;
        end %i
        
    elseif ( dim == 2 )
        for i = 1:g
            for j = 1:g
                GaussPoint(n,:) = [x(i), x(j)];
                GaussWeight(n)  = w(i) * w(j);
                n = n + 1;
            end %j
        end %i
        
    else
        for i = 1:g
            for j = 1:g
                for k = 1:g
                    GaussPoint(n,:) = [ x(i), x(j), x(k) ];
                    GaussWeight(n)  = w(i) * w(j) * w(k);
                    n = n + 1;
                end %k
            end %j
        end %i
    
    end %if
    
    X = GaussPoint;
    W = GaussWeight;
 
end


